// Acorn is a tiny, fast JavaScript parser written in JavaScript.
//
// Acorn was written by Marijn Haverbeke, Ingvar Stepanyan, and
// various contributors and released under an MIT license.
//
// Git repositories for Acorn are available at
//
//     http://marijnhaverbeke.nl/git/acorn
//     https://github.com/acornjs/acorn.git
//
// Please use the [github bug tracker][ghbt] to report issues.
//
// [ghbt]: https://github.com/acornjs/acorn/issues
//
// This file defines the main parser interface. The library also comes
// with a [error-tolerant parser][dammit] and an
// [abstract syntax tree walker][walk], defined in other files.
//
// [dammit]: acorn_loose.js
// [walk]: util/walk.js


var Parser = require('./state').Parser;
require('./location')
require('./parseutil')
require('./scope')
require('./statement')
require('./expression')
require('./lval')
module.exports.Parser = require('./state').Parser;
module.exports.plugins = require('./state').plugins;
module.exports.defaultOptions = require('./options').defaultOptions;
module.exports.Position = require('./locutil').Position;
module.exports.SourceLocation = require('./locutil').SourceLocation;
module.exports.getLineInfo = require('./locutil').getLineInfo;
module.exports.Node = require('./node').Node;
module.exports.TokenType = require('./tokentype').TokenType;
module.exports.tokTypes = require('./tokentype').types;
module.exports.keywordTypes = require('./tokentype').keywords;
module.exports.TokContext = require('./tokencontext').TokContext;
module.exports.tokContexts = require('./tokencontext').types;
module.exports.Node = require('./node').Node;
module.exports.isIdentifierChar = require('./identifier').isIdentifierChar;
module.exports.isIdentifierStart = require('./identifier').isIdentifierStart;
module.exports.Token = require('./tokenize').Token;
module.exports.isNewLine = require('./whitespace').isNewLine;
module.exports.lineBreak = require('./whitespace').lineBreak;
module.exports.lineBreakG = require('./whitespace').lineBreakG;
module.exports.nonASCIIwhitespace = require('./whitespace').nonASCIIwhitespace;

const version = "5.5.3"

// The main exported interface (under `self.acorn` when in the
// browser) is a `parse` function that takes a code string and
// returns an abstract syntax tree as specified by [Mozilla parser
// API][api].
//
// [api]: https://developer.mozilla.org/en-US/docs/SpiderMonkey/Parser_API

function parse(input, options) {
  return new Parser(options, input).parse()
}

// This function tries to parse a single expression at a given
// offset in a string. Useful for parsing mixed-language formats
// that embed JavaScript expressions.

function parseExpressionAt(input, pos, options) {
  let p = new Parser(options, input, pos)
  p.nextToken()
  return p.parseExpression()
}

// Acorn is organized as a tokenizer and a recursive-descent parser.
// The `tokenizer` export provides an interface to the tokenizer.

function tokenizer(input, options) {
  return new Parser(options, input)
}

// This is a terrible kludge to support the existing, pre-ES6
// interface where the loose parser module retroactively adds exports
// to this module.
let parse_dammit, LooseParser, pluginsLoose // eslint-disable-line camelcase
function addLooseExports(parse, Parser, plugins) {
  parse_dammit = parse // eslint-disable-line camelcase
  LooseParser = Parser
  pluginsLoose = plugins
}
module.exports.parse_dammit = parse_dammit;
module.exports.LooseParser = LooseParser;
module.exports.pluginsLoose = pluginsLoose;
module.exports.addLooseExports = addLooseExports;
module.exports.tokenizer = tokenizer;
module.exports.parseExpressionAt = parseExpressionAt;
module.exports.parse = parse;
module.exports.version = version;